/*
 * Copyright 2016 (c) Anna Schumaker.
 */
#ifndef OCARINA_CORE_PLAYLISTS_GENERIC_H
#define OCARINA_CORE_PLAYLISTS_GENERIC_H
#include <core/playlists/playlist.h>

enum playlist_save_flags {
	PL_SAVE_FLAGS  = (1 << 0), /* Save playlist random and sort data. */
	PL_SAVE_ITER   = (1 << 1), /* Save playlist iterator position.    */
	PL_SAVE_TRACKS = (1 << 2), /* Save playlist tracks.               */
};
#define PL_SAVE_METADATA (PL_SAVE_FLAGS  | PL_SAVE_ITER)
#define PL_SAVE_ALL      (PL_SAVE_TRACKS | PL_SAVE_METADATA)

struct playlist_callbacks {
	/* Called to notify that a playlist has been sorted. */
	void (*pl_cb_sorted)(struct playlist *);
};


/* Called to set playlist callbacks. */
void playlist_generic_set_callbacks(struct playlist_callbacks *);

/* Generic playlist init function. */
void playlist_generic_init(struct playlist *, struct queue_ops *);

/* Generic playlist save function. */
void playlist_generic_save(struct playlist *, struct file *, unsigned int);

/* Generic playlist load function. */
void playlist_generic_load(struct playlist *, struct file *, unsigned int);

/* Generic playlist can-select function. */
bool playlist_generic_can_select(struct playlist *);

/* Generic playlist clear operation. */
void playlist_generic_clear(struct playlist *);

/* Generic playlist add track operations. */
bool playlist_generic_add_track(struct playlist *, struct track *);
bool playlist_generic_add_track_front(struct playlist *, struct track *);

/* Generic playlist remove track operation. */
bool playlist_generic_remove_track(struct playlist *, struct track *);

/* Generic playlist set_random operation. */
void playlist_generic_set_random(struct playlist *, bool);

/* Generic playlist sorting operations. */
void playlist_generic_sort(struct playlist *, enum compare_t);
void playlist_generic_resort(struct playlist *);

/* Generic playlist next track operation. */
struct track *playlist_generic_next(struct playlist *);

#endif /* OCARINA_CORE_PLAYLISTS_GENERIC_H */
